/*
  Copyright (c) 2014 by I.Kato @ BS21 Lab
  Serial EEPROM Yaminabe Shield (BS21LAB-017) library for Arduino
  
  Based: 
    Copyright (c) 2010 by arms22 (arms22 at gmail.com)
    Microchip 23x256 SPI SRAM library for Arduino
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*/

#include "SPIEEPROM64K.h"


// シリアル EEPROM 闇鍋シールド (BS21LAB-017) 用ライブラリ
// ※ 本ライブラリは、arms22氏作成の SPISRAM ライブラリをベースにしています。
// SPI 通信のみ。
// 動作確認したのは microchip 製 25AA512 のみです。他でも使用できるかは不明です。


// 注意：連続 write 関数ですが、25AA512 では、128バイト毎のページ単位の範囲内でしか、連続書き込みできません。
//       もし、ページをまたぐ様な連続書き込みを行おうとすると、同一ページ内の先頭に戻ります。
//       例：0x20F0 に32バイト連続書き込みを行おうとすると、最初の16バイトは 0x20F0～0x20FF に書き込まれますが、
//           残りの16バイトは 0x2080～0x208F に書き込まれてしまいます。これは 0x2080～0x20FF で１つのページになるからです。
//       この為、ページをまたいで書き込む時は、ページ単位に、連続 write 関数を分離する必要が有ります。
//       そのページサイズも、25AA512 以外では違うサイズかもしれません。スペックシートで確認して下さい。

// 注意：１バイト write 関数、および、連続 write 関数を呼び出す時は、その直前に、writeEnable 関数を呼び出して、
//       書き込み許可状態にする必要が有ります。書き込み禁止状態のままでは、write 関数を呼び出しても EEPROM には書き込まれません。
//       また、１バイト write 関数、もしくは、連続 write 関数を１回でも呼び出すと、再び、書き込み禁止状態に戻ります。


// INSTRUCTION SET
#define READ	0x03			// Read data from memory
#define WRITE	0x02			// Write data to memory
#define RDSR	0x05			// Read Status register
#define WRSR	0x01			// Write Status register
#define WREN	0x06			// Write Enable
#define WRDI	0x04			// Write Disable


// コンストラクタ
// CSピンを指定する必要が有ります
SPIEEPROM64K::SPIEEPROM64K (byte ncsPin)
	: _ncsPin(ncsPin)
{
}

// 初期化
// SPI初期化と CSピンを出力モードにします
void SPIEEPROM64K::init (void)
{
	SPI.begin();

	SPI.setBitOrder     (MSBFIRST);
	SPI.setClockDivider (SPI_CLOCK_DIV4);
	SPI.setDataMode     (SPI_MODE0);

	// Set CS to high so a connected chip will be "deselected" by default
	digitalWrite (_ncsPin, HIGH);

	// When the CS pin is set as OUTPUT, it can be used as
	// a general purpose output port (it doesn't influence
	// SPI operations).
	pinMode (_ncsPin, OUTPUT);
}

// １バイト読み出し
byte SPIEEPROM64K::read (unsigned int address)
{
	_buf[0] = READ;
	_buf[1] = address >> 8;
	_buf[2] = address & 0xff;
	_buf[3] = 0x00;
	return transfer(4);
}

// 複数バイト読み出し
void SPIEEPROM64K::read (unsigned int address, byte *buffer, unsigned int size)
{
	unsigned int i;
	select();
	SPI.transfer (READ);
	SPI.transfer (address >> 8);
	SPI.transfer (address & 0xff);
	for (i = 0; i < size; i++){
		*buffer++ = SPI.transfer(0);
	}
	deselect();
}

// １バイト書き込み
void SPIEEPROM64K::write (unsigned int address, byte data)
{
	_buf[0] = WRITE;
	_buf[1] = address >> 8;
	_buf[2] = address & 0xff;
	_buf[3] = data;
	transfer(4);
}

// 複数バイト書き込み
// 25AA512 では 128バイトのページをまたがった書き込みはできません（もし超えるとページの先頭に戻ります）。
// そのページサイズも、25AA512 以外ではサイズが違う可能性がありますので、スペックシートで確認して下さい。
void SPIEEPROM64K::write (unsigned int address, byte *buffer, unsigned int size)
{
	unsigned int i;
	select();
	SPI.transfer (WRITE);
	SPI.transfer (address >> 8);
	SPI.transfer (address & 0xff);
	for (i = 0; i < size; i++) {
		SPI.transfer (*buffer++);
	}
	deselect();
}

// 書き込み許可
// Write コマンドを実行すると、再度、書き込み禁止になりますので、
// Write の前に、この関数を呼び出す必要が有ります。
void SPIEEPROM64K::writeEnable (void)
{
	_buf[0] = WREN;
	transfer(1);
}

// 書き込み禁止
void SPIEEPROM64K::writeDisable (void)
{
	_buf[0] = WRDI;
	transfer(1);
}

// CSピンを選択
void SPIEEPROM64K::select (void)
{
	digitalWrite (_ncsPin, LOW);
}

// CSピンを解放
void SPIEEPROM64K::deselect (void)
{
	digitalWrite (_ncsPin, HIGH);
}

// SPI通信
byte SPIEEPROM64K::transfer (byte length)
{
	byte ret, i;
	select();
	for (i = 0; i < length; i++){
		ret = SPI.transfer (_buf[i]);
	}
	deselect();
	return ret;
}

