/*
  Copyright (c) 2014 by I.Kato @ BS21 Lab
  Serial EEPROM Yaminabe Shield (BS21LAB-017) library for Arduino
  
  Based: 
    Copyright (c) 2010 by arms22 (arms22 at gmail.com)
    Microchip 23x256 SPI SRAM library for Arduino
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*/

#ifndef SPIEEPROM64K_H
#define SPIEEPROM64K_H

#include <SPI.h>

// シリアル EEPROM 闇鍋シールド (BS21LAB-017) 用ライブラリ
// ※ 本ライブラリは、arms22氏作成の SPISRAM ライブラリをベースにしています。
// SPI 通信のみ。
// 動作確認したのは microchip 製 25AA512 のみです。他でも使用できるかは不明です。

// 注意：連続 write 関数ですが、25AA512 では、128バイト毎のページ単位の範囲内でしか、連続書き込みできません。
//       もし、ページをまたぐ様な連続書き込みを行おうとすると、同一ページ内の先頭に戻ります。
//       例：0x20F0 に32バイト連続書き込みを行おうとすると、最初の16バイトは 0x20F0～0x20FF に書き込まれますが、
//           残りの16バイトは 0x2080～0x208F に書き込まれてしまいます。これは 0x2080～0x20FF で１つのページになるからです。
//       この為、ページをまたいで書き込む時は、ページ単位に、連続 write 関数を分離する必要が有ります。
//       そのページサイズも、25AA512 以外では違うサイズかもしれません。スペックシートで確認して下さい。

// 注意：１バイト write 関数、および、連続 write 関数を呼び出す時は、その直前に、writeEnable 関数を呼び出して、
//       書き込み許可状態にする必要が有ります。書き込み禁止状態のままでは、write 関数を呼び出しても EEPROM には書き込まれません。
//       また、１バイト write 関数、もしくは、連続 write 関数を１回でも呼び出すと、再び、書き込み禁止状態に戻ります。


class SPIEEPROM64K
{
public:
	struct MemCell {
		SPIEEPROM64K &device;
		unsigned int address;
		MemCell &operator= (byte data) {
			device.write (address, data);
			return *this;
		}
		operator byte() {
			return device.read (address);
		}
		MemCell (SPIEEPROM64K &d, unsigned int a) : device(d), address(a) {};
		~MemCell(){};
	};
	MemCell operator[] (unsigned int address) {
		MemCell memcell (*this, address);
		return memcell;
	}
public:
	SPIEEPROM64K (byte ncsPin);
	void init (void);  // 初期化
	byte read (unsigned int address); // １バイト読み出し
	void read (unsigned int address, byte *buffer, unsigned int size); // 連続バイト読み出し
	void write(unsigned int address, byte data); // １バイト書き込み
	void write(unsigned int address, byte *buffer, unsigned int size); // 連続バイト書き込み ※上記注意事項を参照
public:
	void writeEnable (void);  // 書き込み許可状態にする
	void writeDisable(void);  // 書き込み禁止状態にする
private:
	byte _ncsPin;
	byte _buf[4];
	void select  (void);
	void deselect(void);
	byte transfer(byte length);
};

#endif
